use std::ops::Deref;

use rocket::{
    http::{CookieJar, Status},
    State,
};
use rocket_dyn_templates::{context, Template};

use crate::{
    cookies::LANG_COOKIE,
    i18n::DEFAULT as DEFAULT_LANG,
    models::{DatabaseError, SessionUser, Settings},
    utils::AcceptLanguage, DATABASE,
};

#[get("/users/<user>")]
pub async fn get_user(
    user: String,
    cookies: &CookieJar<'_>,
    settings: &State<Settings>,
    accept_language: AcceptLanguage,
) -> Result<Template, Status> {
    let profile_user = match DATABASE.get_user_by_name(&user) {
        Ok(profile_user) => profile_user,
        Err(DatabaseError::Rusqlite(rusqlite::Error::QueryReturnedNoRows)) => {
            return Err(Status::NotFound)
        }
        Err(error) => {
            eprintln!("{:?}", error);
            return Err(Status::InternalServerError);
        }
    };
    Ok(Template::render(
        "user",
        context! {
            profile_user,
            submissions: DATABASE.get_submissions_by_user_name(&user).unwrap(),
            settings: settings.deref(),
            lang: cookies
                .get(LANG_COOKIE)
                .map(|cookie| vec![cookie.value().to_owned()])
                .or(accept_language.0)
                .unwrap_or_else(|| vec![DEFAULT_LANG.to_owned()]),
            user: SessionUser::get(cookies).await.unwrap(),
        },
    ))
}
